#import "TUArchiveController.h"
#import "TUController.h"
#import "TUListView.h"
#import "TUEncodingPopUp.h"


@implementation TUArchiveController

-(id)initWithFilename:(NSString *)filename controller:(TUController *)controller
{
	if(self=[super init])
	{
		view=nil;
		cancelled=NO;
		ignoreall=NO;
		selected_encoding=0;

		waitview=nil;
		progressview=nil;
		errorview=nil;
		openerrorview=nil;
		passwordview=nil;
		encodingview=nil;

		archivename=[filename retain];
		maincontroller=controller;

		NSString *directory=[filename stringByDeletingLastPathComponent];
		NSString *tmpdir=[NSString stringWithFormat:@".tmp%04x%04x%04x",rand()&0xffff,rand()&0xffff,rand()&0xffff];
		tmpdest=[[directory stringByAppendingPathComponent:tmpdir] retain];
		defaultname=[[[filename lastPathComponent] stringByDeletingPathExtension] retain];
		pauselock=[[NSLock alloc] init];
	}
	return self;
}

-(void)dealloc
{
	[archive release];
	[tmpdest release];
	[defaultname release];

	[view release];
	[pauselock release];

	[waitview release];
	[progressview release];
	[errorview release];
	[openerrorview release];
	[passwordview release];
	[encodingview release];

	[super dealloc];
}


-(void)wait
{
	[self setupWaitView];
}

-(void)go
{
	[self setupProgressView];
	[NSThread detachNewThreadSelector:@selector(extract) toTarget:self withObject:nil];
}

-(void)stop
{
	[[maincontroller listView] removeSubview:view];
}



-(void)extract
{
	NSAutoreleasePool *pool=[[NSAutoreleasePool alloc] init];

	archive=[[XADArchive alloc] initWithFile:archivename];
	if(!archive)
	{
		[self performSelectorOnMainThread:@selector(extractOpenFailed) withObject:nil waitUntilDone:NO];
		[pool release];
		return;
	}

	[archive setDelegate:self];
	[archive setDetector:[UniversalDetector detector]];
	[archive setWinnyVirusWorkaround:YES];

	if([archive isEncrypted])
	{
		[pauselock lock];
		[self performSelectorOnMainThread:@selector(setupPasswordView) withObject:nil waitUntilDone:NO];
		[pauselock lock];
		[pauselock unlock];

		if(cancelled)
		{
			[self performSelectorOnMainThread:@selector(extractFailed) withObject:nil waitUntilDone:NO];
			[pool release];
			return;
		}
	
		[self performSelectorOnMainThread:@selector(setupProgressView) withObject:nil waitUntilDone:NO];
	}

	firstprogress=YES;
	BOOL res;

	if([archive numberOfEntries]==1)
	{
		NSString *ext=[[[archive nameOfEntry:0] pathExtension] lowercaseString];
		if([ext isEqual:@"tar"]||[ext isEqual:@"sit"]||[ext isEqual:@"sea"])
		{
			[defaultname release];
			defaultname=[[[[archive nameOfEntry:0] lastPathComponent] stringByDeletingPathExtension]  retain];

			res=[archive extractArchiveEntry:0 to:tmpdest];
		}
		else res=[archive extractTo:tmpdest];
	}
	else res=[archive extractTo:tmpdest];

	if(!res)
	{
		[self performSelectorOnMainThread:@selector(extractFailed) withObject:nil waitUntilDone:NO];
		[pool release];
		return;
	}

	[self performSelectorOnMainThread:@selector(extractFinished) withObject:nil waitUntilDone:NO];
	[pool release];
}

-(void)extractFinished
{
	NSFileManager *fm=[NSFileManager defaultManager];
	NSArray *files=[fm directoryContentsAtPath:tmpdest];

	if(files)
	{
		if([files count]==1)
		{
			NSString *filename=[files objectAtIndex:0];
			NSString *src=[tmpdest stringByAppendingPathComponent:filename];
			NSString *directory=[[archive filename] stringByDeletingLastPathComponent];
			NSString *dest=[self findUniqueDestinationWithDirectory:directory andFilename:filename];
			[fm movePath:src toPath:dest handler:nil];
			[fm removeFileAtPath:tmpdest handler:nil];
		}
		else
		{
			NSString *directory=[[archive filename] stringByDeletingLastPathComponent];
			NSString *dest=[self findUniqueDestinationWithDirectory:directory andFilename:defaultname];
			[fm movePath:tmpdest toPath:dest handler:nil];
		}
	}

	[maincontroller archiveFinished:self];
}

-(void)extractOpenFailed
{
	[self setupOpenErrorView];
}

-(void)extractFailed
{
	NSFileManager *fm=[NSFileManager defaultManager];
	[fm removeFileAtPath:tmpdest handler:nil];

	[maincontroller archiveFinished:self];
}

-(NSString *)findUniqueDestinationWithDirectory:(NSString *)directory andFilename:(NSString *)filename
{
	NSString *basename=[filename stringByDeletingPathExtension];
	NSString *extension=[filename pathExtension];
	if([extension length]) extension=[@"." stringByAppendingString:extension];

	NSString *destination=[directory stringByAppendingPathComponent:filename];
	int n=1;

	while([[NSFileManager defaultManager] fileExistsAtPath:destination])
	{
		destination=[directory stringByAppendingPathComponent:
		[NSString stringWithFormat:@"%@-%d%@",basename,n++,extension]];
	}

	return destination;
}




-(BOOL)archiveExtractionShouldStop:(XADArchive *)sender { return cancelled; }

-(NSStringEncoding)archive:(XADArchive *)archive encodingForName:(const char *)bytes guess:(NSStringEncoding)guess confidence:(float)confidence
{
	NSStringEncoding encoding=[[NSUserDefaults standardUserDefaults] integerForKey:@"filenameEncoding"];
	int threshold=[[NSUserDefaults standardUserDefaults] integerForKey:@"autoDetectionThreshold"];

	if(encoding) return encoding;
	else if(selected_encoding) return selected_encoding;
	else if(confidence*100<threshold)
	{
		XADAction action=[self displayEncodingSelectorForBytes:bytes encoding:guess];
		if(action==XADAbort) cancelled=YES;
		return selected_encoding;
	}
	else return guess;
}

-(void)archive:(XADArchive *)sender extractionOfEntryWillStart:(int)n
{
	NSString *name=[sender nameOfEntry:n];
	if(name) [namefield performSelectorOnMainThread:@selector(setStringValue:) withObject:name waitUntilDone:NO];
}

-(void)archive:(XADArchive *)sender extractionProgressBytes:(xadSize)bytes of:(xadSize)total
{
	if(firstprogress)
	{
		[self performSelectorOnMainThread:@selector(progressStart:)
		withObject:[NSNumber numberWithUnsignedLongLong:total] waitUntilDone:NO];
		firstprogress=NO;
	}
	else
	{
		[self performSelectorOnMainThread:@selector(progressUpdate:)
		withObject:[NSNumber numberWithUnsignedLongLong:bytes] waitUntilDone:NO];
	}
}

-(void)archive:(XADArchive *)sender immediateExtractionInputProgressBytes:(xadSize)bytes of:(xadSize)total
{
	if(firstprogress)
	{
		[self performSelectorOnMainThread:@selector(progressStart:)
		withObject:[NSNumber numberWithUnsignedLongLong:total] waitUntilDone:NO];
		firstprogress=NO;
	}
	else
	{
		[self performSelectorOnMainThread:@selector(progressUpdate:)
		withObject:[NSNumber numberWithUnsignedLongLong:bytes] waitUntilDone:NO];
	}
}

-(void)progressStart:(NSNumber *)total
{
	[actionfield setStringValue:[NSString stringWithFormat:@"Extracting \"%@\"",[[archive filename] lastPathComponent]]];
	[progress setIndeterminate:NO];
	[progress setDoubleValue:0];
	[progress setMaxValue:[total unsignedLongLongValue]];
}

-(void)progressUpdate:(NSNumber *)bytes
{
	[progress setDoubleValue:[bytes unsignedLongLongValue]];
}



-(XADAction)archive:(XADArchive *)archive nameDecodingDidFailForEntry:(int)n bytes:(const char *)bytes
{
	return [self displayEncodingSelectorForBytes:bytes encoding:0];
}

-(XADAction)archive:(XADArchive *)sender extractionOfEntryDidFail:(int)n error:(XADError)error
{
	return [self displayError:
		[NSString stringWithFormat:@"Could not extract the file \"%@\":\n%@",
		[sender nameOfEntry:n],[archive describeError:error]]
	];
}

-(XADAction)archive:(XADArchive *)sender extractionOfResourceForkForEntryDidFail:(int)n error:(XADError)error
{
	return [self displayError:
		[NSString stringWithFormat:@"Could not extract the resource fork for the file \"%@\":\n%@",
		[sender nameOfEntry:n],[archive describeError:error]]
	];
}

-(XADAction)displayError:(NSString *)error
{
	if(ignoreall) return XADSkip;

	[pauselock lock];
	[self performSelectorOnMainThread:@selector(setupErrorView:) withObject:error waitUntilDone:NO];
	[pauselock lock];
	[pauselock unlock];

	[self performSelectorOnMainThread:@selector(setDisplayedView:) withObject:progressview waitUntilDone:NO];

	return erroraction;
}

-(XADAction)displayEncodingSelectorForBytes:(const char *)bytes encoding:(NSStringEncoding)encoding
{
	selected_encoding=encoding;
	name_bytes=bytes;

	[pauselock lock];
	[self performSelectorOnMainThread:@selector(setupEncodingView) withObject:nil waitUntilDone:NO];
	[pauselock lock];
	[pauselock unlock];

	[self performSelectorOnMainThread:@selector(setDisplayedView:) withObject:progressview waitUntilDone:NO];

	return encodingaction;
}






-(IBAction)cancelWait:(id)sender
{
	[maincontroller archiveCancelled:self];
	[sender setEnabled:NO];
}

-(IBAction)cancelExtraction:(id)sender
{
	cancelled=YES;
	[sender setEnabled:NO];
}

-(IBAction)stopAfterError:(id)sender
{
	erroraction=XADAbort;
	[pauselock unlock];
}

-(IBAction)continueAfterError:(id)sender
{
	erroraction=XADSkip;
	ignoreall=([applyallcheck state]==NSOnState?YES:NO);
	[pauselock unlock];
}

-(IBAction)okAfterOpenError:(id)sender
{
	[maincontroller archiveFinished:self];
}

-(IBAction)stopAfterPassword:(id)sender
{
	cancelled=YES;
	[pauselock unlock];
}

-(IBAction)continueAfterPassword:(id)sender
{
	[archive setPassword:[passwordfield stringValue]];
	[pauselock unlock];
}

-(IBAction)continueAfterEncoding:(id)sender
{
	encodingaction=XADRetry;
	[pauselock unlock];
}

-(IBAction)stopAfterEncoding:(id)sender
{
	encodingaction=XADAbort;
	[pauselock unlock];
}

-(IBAction)selectEncoding:(id)sender
{
	selected_encoding=[encodingpopup selectedTag];
	[encodingfield setStringValue:[NSString stringWithCString:name_bytes encoding:selected_encoding]];
}



-(void)setupWaitView
{
	if(!waitview)
	{
		NSNib *nib=[[[NSNib alloc] initWithNibNamed:@"WaitView" bundle:nil] autorelease];
		[nib instantiateNibWithOwner:self topLevelObjects:nil];
	}

	[waitfield setStringValue:[archivename lastPathComponent]];

	NSImage *icon=[[NSWorkspace sharedWorkspace] iconForFile:archivename];
	[icon setSize:[waiticon frame].size];
	[waiticon setImage:icon];

	[self setDisplayedView:waitview];
}

-(void)setupProgressView
{
	if(!progressview)
	{
		NSNib *nib=[[[NSNib alloc] initWithNibNamed:@"ProgressView" bundle:nil] autorelease];
		[nib instantiateNibWithOwner:self topLevelObjects:nil];
	}

	[actionfield setStringValue:[NSString stringWithFormat:@"Preparing to extract \"%@\"",[archivename lastPathComponent]]];
	[namefield setStringValue:@""];

	NSImage *icon=[[NSWorkspace sharedWorkspace] iconForFile:archivename];
	[icon setSize:[progressicon frame].size];
	[progressicon setImage:icon];

	[progress setIndeterminate:YES];
	[progress startAnimation:self];

	[self setDisplayedView:progressview];
}

-(void)setupErrorView:(NSString *)error
{
	if(!errorview)
	{
		NSNib *nib=[[[NSNib alloc] initWithNibNamed:@"ErrorView" bundle:nil] autorelease];
		[nib instantiateNibWithOwner:self topLevelObjects:nil];
	}

	[errorfield setStringValue:error];
	[self setDisplayedView:errorview];
}

-(void)setupOpenErrorView
{
	if(!openerrorview)
	{
		NSNib *nib=[[[NSNib alloc] initWithNibNamed:@"OpenErrorView" bundle:nil] autorelease];
		[nib instantiateNibWithOwner:self topLevelObjects:nil];
	}

	[openerrorfield setStringValue:[NSString stringWithFormat:@"The contents of the file \"%@\" can not be extracted with this program.",
	[archivename lastPathComponent]]];
	[self setDisplayedView:openerrorview];
}

-(void)setupPasswordView
{
	if(!passwordview)
	{
		NSNib *nib=[[[NSNib alloc] initWithNibNamed:@"PasswordView" bundle:nil] autorelease];
		[nib instantiateNibWithOwner:self topLevelObjects:nil];
	}

	NSImage *icon=[[NSWorkspace sharedWorkspace] iconForFile:archivename];
	[icon setSize:[passwordicon frame].size];
	[passwordicon setImage:icon];

	[self setDisplayedView:passwordview];
	[[passwordfield window] makeFirstResponder:passwordfield];
}

-(void)setupEncodingView
{
	if(!encodingview)
	{
		NSNib *nib=[[[NSNib alloc] initWithNibNamed:@"EncodingView" bundle:nil] autorelease];
		[nib instantiateNibWithOwner:self topLevelObjects:nil];
	}

	NSImage *icon=[[NSWorkspace sharedWorkspace] iconForFile:archivename];
	[icon setSize:[encodingicon frame].size];
	[encodingicon setImage:icon];

	[encodingpopup buildEncodingListMatchingBytes:name_bytes];
	if(selected_encoding) [encodingpopup selectItemWithTag:selected_encoding];

	[self selectEncoding:self];

	[self setDisplayedView:encodingview];
}



-(void)setDisplayedView:(NSView *)dispview
{
	NSRect frame=[dispview frame];
	[dispview setAutoresizingMask:NSViewWidthSizable|NSViewMinYMargin];

	if(view)
	{
		NSEnumerator *enumerator=[[view subviews] objectEnumerator];
		NSView *subview;
		while(subview=[enumerator nextObject]) [subview removeFromSuperview];

		NSSize viewsize=[view frame].size;
		NSRect newframe=NSMakeRect(0,viewsize.height-frame.size.height,viewsize.width,frame.size.height);
		[dispview setFrame:newframe];
		[view addSubview:dispview];
		[[maincontroller listView] setHeight:frame.size.height forView:view];
	}
	else
	{
		view=[[NSView alloc] init];
		[view setAutoresizesSubviews:YES];

		NSRect newframe=NSMakeRect(0,0,frame.size.width,frame.size.height);
		[view setFrame:newframe];
		[dispview setFrame:newframe];
		[view addSubview:dispview];

		[[maincontroller listView] addSubview:view];
	}

}

@end
